@@ -10,32 +10,9 @@ The Map and Tod classes subclass FitsArray.
 
 These classes are useful to load, manipulate and save FITs files.
 They also contain specialised display methods.
+
 """
-from __future__ import division
-
-try:
-    import kapteyn.maputils as km
-except ImportError:
-    km = None
-try:
-    import matplotlib
-    import matplotlib.pyplot as mp
-except ImportError:
-    pass
-import numpy as np
-import os
-import pickle
-import StringIO
-import scipy.stats
-import sys
-import time
-import uuid
-
-try:
-    import ds9
-except ImportError:
-    ds9 = None
-
+from __future__ import absolute_import, division, print_function
 from astropy.io import fits as pyfits
 from functools import reduce
 try:
@@ -44,10 +21,19 @@ except ImportError:
     from pyoperators.memory import allocate as empty
 from pyoperators.utils import isscalarlike
 from pyoperators.utils.mpi import MPI
-
 from .mpiutils import read_fits, write_fits
 from .quantities import Quantity
 from .wcsutils import create_fitsheader_for, has_wcs
+import io
+import numpy as np
+import os
+import pickle
+import scipy.stats
+import sys
+import time
+import uuid
+if sys.version_info.major > 2:
+    basestring = str
 
 __all__ = ['FitsArray', 'Map', 'Tod']
 
@@ -107,22 +93,22 @@ class FitsArray(Quantity):
     comm : mpi4py.Comm, optional
         MPI communicator specifying to which processors the FITS file
         should be distributed.
-    """
 
+    """
     _header = None
 
     def __new__(cls, data, header=None, unit=None, derived_units=None,
                 dtype=None, copy=True, order='C', subok=False, ndmin=0,
                 comm=None):
 
-        if isinstance(data, (str, unicode)):
+        if isinstance(data, basestring):
 
             if comm is None:
-                comm = MPI.COMM_WORLD
+                comm = MPI.COMM_SELF
 
             try:
                 buf = pyfits.open(data)['derived_units'].data
-                derived_units = pickle.loads(str(buf.data))
+                derived_units = pickle.loads(buf.data)
             except (ImportError, KeyError):
                 pass
 
@@ -204,7 +190,7 @@ class FitsArray(Quantity):
         if self._header is not None:
             typename = np.dtype(dtype).name
             result._header = self._header.copy()
-            result._header['BITPIX'] = pyfits.PrimaryHDU.ImgCode[typename]
+            result._header['BITPIX'] = pyfits.DTYPE2BITPIX[typename]
         return result
 
     @classmethod
@@ -269,7 +255,7 @@ class FitsArray(Quantity):
 
         header = self.header.copy()
         if comm is None:
-            comm = MPI.COMM_WORLD
+            comm = MPI.COMM_SELF
 
         if len(self._unit) != 0:
             header['BUNIT'] = self.unit
@@ -278,7 +264,7 @@ class FitsArray(Quantity):
         if not self.derived_units:
             return
         if comm is None or comm.rank == 0:
-            buf = StringIO.StringIO()
+            buf = io.BytesIO()
             pickle.dump(self.derived_units, buf, pickle.HIGHEST_PROTOCOL)
             data = np.frombuffer(buf.getvalue(), np.uint8)
             write_fits(filename, data, None, True, 'derived_units',
@@ -294,6 +280,8 @@ class FitsArray(Quantity):
         -------
         >>> Map(np.arange(100).reshape((10,10))).imsave('plot.png')
         """
+        import matplotlib
+        import matplotlib.pyplot as mp
         is_interactive = matplotlib.is_interactive()
         matplotlib.interactive(False)
         dpi = 80.
@@ -336,6 +324,7 @@ class FitsArray(Quantity):
         **keywords : additional are keywords passed to matplotlib's imshow.
 
         """
+        import matplotlib.pyplot as mp
         if np.iscomplexobj(self):
             data = abs(self.magnitude)
         else:
@@ -423,13 +412,15 @@ class FitsArray(Quantity):
         >>> d.set('exit')
 
         """
-        if ds9 is None:
+        try:
+            import pyds9
+        except ImportError:
             raise ImportError('The library pyds9 has not been installed.')
         import xpa
 
         id = None
         if not new:
-            list = ds9.ds9_targets()
+            list = pyds9.ds9_targets()
             if list is not None:
                 id = list[-1]
 
@@ -440,8 +431,9 @@ class FitsArray(Quantity):
             if 'scale' not in keywords:
                 keywords['scale'] = ('scope local', 'mode 99.5')
 
-            if origin == 'upper' or \
-               'orient' not in keywords and self.origin == 'upper':
+            if origin is None:
+                origin = getattr(self, 'origin', 'lower')
+            if origin == 'upper' and 'orient' not in keywords:
                 keywords['orient'] = 'y'
 
             wait = 10
@@ -479,7 +471,7 @@ class FitsArray(Quantity):
                 raise ValueError('No active ds9 running for target: %s' % list)
 
         # get ds9 instance with given id
-        d = ds9.ds9(id_)
+        d = pyds9.DS9(id_)
 
         # load array
         input = self.view(np.ndarray)
@@ -579,10 +571,10 @@ class Map(FitsArray):
         if not subok and type(result) is not cls:
             result = result.view(cls)
 
-        if isinstance(data, (str, unicode)):
+        if isinstance(data, basestring):
 
             if comm is None:
-                comm = MPI.COMM_WORLD
+                comm = MPI.COMM_SELF
 
             if 'DISPORIG' in result.header:
                 if origin is None:
@@ -704,6 +696,11 @@ class Map(FitsArray):
     def imshow(self, mask=None, new_figure=True, title=None, xlabel='X',
                ylabel='Y', interpolation='nearest', origin=None, colorbar=True,
                percentile=0, **keywords):
+        try:
+            import kapteyn.maputils as km
+        except ImportError:
+            km = None
+        import matplotlib.pyplot as mp
         if mask is None and self.coverage is not None:
             mask = self.coverage <= 0
 
@@ -772,7 +769,7 @@ class Map(FitsArray):
 
     def save(self, filename, comm=None):
         if comm is None:
-            comm = MPI.COMM_WORLD
+            comm = MPI.COMM_SELF
 
         self.header['DISPORIG'] = self.origin, 'Map display convention'
         FitsArray.save(self, filename, comm=comm)
@@ -865,10 +862,10 @@ class Tod(FitsArray):
         if mask is np.ma.nomask:
             mask = None
 
-        if mask is None and isinstance(data, (str, unicode)):
+        if mask is None and isinstance(data, basestring):
 
             if comm is None:
-                comm = MPI.COMM_WORLD
+                comm = MPI.COMM_SELF
 
             try:
                 mask, junk = read_fits(data, 'mask', comm)
@@ -906,7 +903,7 @@ class Tod(FitsArray):
                 mask = np.asarray(mask, np.bool8)
 
         # handle the scalar case
-        if np.rank(mask) == 0:
+        if mask.ndim == 0:
             if self._mask is None:
                 func = np.zeros if mask == 0 else np.ones
                 self._mask = func(self.shape, dtype=np.bool8)
@@ -981,7 +978,7 @@ class Tod(FitsArray):
         if self.ndim == 0:
             return output
         output += ' ('
-        if np.rank(self) > 1:
+        if self.ndim > 1:
             output += str(self.shape[-2])+' detector'
             if self.shape[-2] > 1:
                 output += 's'
@@ -993,7 +990,7 @@ class Tod(FitsArray):
 
     def save(self, filename, comm=None):
         if comm is None:
-            comm = MPI.COMM_WORLD
+            comm = MPI.COMM_SELF
 
         FitsArray.save(self, filename, comm=comm)
         if self.mask is not None:
