@@ -1,14 +1,5 @@
 #encoding: utf-8
-from __future__ import division
-
-import functools
-import inspect
-import numpy as np
-import operator
-import pyoperators
-import scipy.constants
-
-from itertools import izip
+from __future__ import absolute_import, division, print_function
 from pyoperators import (
     Operator, BlockDiagonalOperator, Cartesian2SphericalOperator,
     CompositionOperator, DenseOperator, DenseBlockDiagonalOperator,
@@ -20,13 +11,22 @@ from pyoperators.memory import empty, ones
 from pyoperators.utils import (
     float_dtype, isscalarlike, operation_assignment, product, strenum,
     tointtuple)
-
 from . import _flib as flib
 from .datatypes import FitsArray, Map
 from .quantities import Quantity, _divide_unit, _multiply_unit
 from .sparse import (
-    FSRMatrix, FSRRotation2dMatrix, FSRRotation3dMatrix, SparseOperator)
+    FSRMatrix, FSRBlockMatrix, FSRRotation2dMatrix, FSRRotation3dMatrix,
+    SparseOperator)
 from .wcsutils import create_fitsheader
+import functools
+import inspect
+import numpy as np
+import operator
+import pyoperators
+import scipy.constants
+import sys
+if sys.version_info.major == 2:
+    from itertools import izip as zip
 
 __all__ = [
     'BlackBodyOperator',
@@ -326,7 +326,7 @@ class ConvolutionTruncatedExponentialOperator(Operator):
     def direct(self, input, output):
         input_, nd, nt, int_all = _ravel_strided(input)
         output_, nd, nt, ont_all = _ravel_strided(output)
-        f = 'trexp_direct_v{0}'.format(input.dtype.itemsize)
+        f = 'trexp_direct_r{0}'.format(input.dtype.itemsize)
         try:
             getattr(flib.operators, f)(
                 input_, nd, nt, int_all,
@@ -337,7 +337,7 @@ class ConvolutionTruncatedExponentialOperator(Operator):
     def transpose(self, input, output):
         input_, nd, nt, int_all = _ravel_strided(input)
         output_, nd, nt, ont_all = _ravel_strided(output)
-        f = 'trexp_transpose_v{0}'.format(input.dtype.itemsize)
+        f = 'trexp_transpose_r{0}'.format(input.dtype.itemsize)
         try:
             getattr(flib.operators, f)(
                 input_, nd, nt, int_all,
@@ -545,7 +545,7 @@ class ProjectionBaseOperator(Operator):
             return
         if not output.flags.contiguous:
             if pyoperators.memory.verbose:
-                print 'Optimize me: Projection output is not contiguous.'
+                print('Optimize me: Projection output is not contiguous.')
             output_ = np.empty_like(output)
         else:
             output_ = output
@@ -564,7 +564,7 @@ class ProjectionBaseOperator(Operator):
             return
         if not input.flags.contiguous:
             if pyoperators.memory.verbose:
-                print 'Optimize me: Projection.T input is not contiguous.'
+                print('Optimize me: Projection.T input is not contiguous.')
             input_ = np.ascontiguousarray(input)
         else:
             input_ = input
@@ -812,15 +812,16 @@ class ProjectionOperator(SparseOperator):
 
     """
     def __init__(self, arg, **keywords):
-        if type(arg) not in (FSRMatrix, FSRRotation2dMatrix,
+        if type(arg) not in (FSRMatrix, FSRBlockMatrix, FSRRotation2dMatrix,
                              FSRRotation3dMatrix):
             raise TypeError('The input sparse matrix type is invalid.')
         self._flib_id = {FSRMatrix: '',
+                         FSRBlockMatrix: '_block',
                          FSRRotation2dMatrix: '_rot2d',
                          FSRRotation3dMatrix: '_rot3d'}[type(arg)]
         SparseOperator.__init__(self, arg, **keywords)
 
-    def canonical_basis_in_kernel(self, out=None,
+    def canonical_basis_in_kernel_old(self, out=None,
                                   operation=operation_assignment):
         """
         Return a boolean array whose values are True if the component of
@@ -829,7 +830,9 @@ class ProjectionOperator(SparseOperator):
         """
         data = self.matrix.data
         shape = self.shapein
-        if not isinstance(self.matrix, FSRMatrix):
+        if shape is None:
+            shape = (self.matrix.shape[1],)
+        elif self.matrix.block_shape[1] > 1:
             shape = shape[:-1]
         if out is None:
             if operation is not operation_assignment:
@@ -845,7 +848,7 @@ class ProjectionOperator(SparseOperator):
             return out
 
         i, m = [_.dtype.itemsize for _ in (data.index, self.matrix)]
-        f = 'fsr{0}_kernel_i{1}_m{2}'.format(self._flib_id, i, m)
+        f = 'fsr{0}_kernel_i{1}_r{2}'.format(self._flib_id, i, m)
         n = product(shape)
         if hasattr(flib.operators, f):
             if operation in (operation_assignment, operator.iand,
@@ -873,6 +876,64 @@ class ProjectionOperator(SparseOperator):
         operation(out, kernel)
         return out
 
+    def canonical_basis_in_kernel(self, out=None,
+                                  operation=operation_assignment):
+        """
+        Return a boolean array whose values are True if the component of
+        the canonical basis is in the operator's kernel.
+
+        """
+        data = self.matrix.data
+        shape = self.shapein
+        if shape is None:
+            shape = (self.matrix.shape[1],)
+        elif self.matrix.block_shape[1] > 1:
+            shape = shape[:-1]
+        if out is None:
+            if operation is not operation_assignment:
+                raise ValueError(
+                    'The array for inplace operation is not specified')
+            out = empty(shape, bool)
+        elif out.dtype != bool:
+            raise TypeError("The keyword 'out' has an invalid type.")
+        elif out.shape != shape:
+            raise ValueError("The keyword 'out' has an incompatible shape.")
+        if data.size == 0:
+            operation(out, True)
+            return out
+
+        itype = data.dtype['index']
+        ftype = data.dtype[1]
+        if itype.type in (np.int8, np.int16, np.int32, np.int64) and \
+           ftype.type in (np.float32, np.float64):
+            if operation in (operation_assignment, operator.iand,
+                             operator.imul):
+                if operation is operation_assignment:
+                    out[...] = True
+                kernel = out
+            else:
+                kernel = np.ones_like(out)
+
+            f = 'fsr_kernel_i{0}'.format(itype.itemsize)
+            func = getattr(flib.sparse, f)
+            func(data.view(np.int8).ravel(), kernel.view(np.int8).ravel(),
+                 self.matrix.ncolmax, product(data.shape[:-1]),
+                 data.strides[-1])
+            if operation in (operation_assignment, operator.iand,
+                             operator.imul):
+                return out
+        else:
+            if isinstance(self.matrix, FSRMatrix):
+                indices = data.index[data.value != 0]
+            elif isinstance(self.matrix, FSRRotation2dMatrix):
+                indices = data.index[(data.r11 != 0) | (data.r21 != 0)]
+            else:
+                indices = data.index[data.r11 != 0]
+            n = product(shape)
+            kernel = np.histogram(indices, n, range=(0, n))[0] == 0
+        operation(out, kernel)
+        return out
+
     def pT1(self, out=None, operation=operation_assignment):
         """
         Return the transpose of the projection over one. In other words,
@@ -880,7 +941,9 @@ class ProjectionOperator(SparseOperator):
 
         """
         shapeout = self.shapein
-        if not isinstance(self.matrix, FSRMatrix):
+        if shapeout is None:
+            shapeout = (self.matrix.shape[1],)
+        elif self.matrix.block_shape[1] > 1:
             shapeout = shapeout[:-1]
         if out is None:
             out = empty(shapeout, self.dtype)
@@ -892,7 +955,7 @@ class ProjectionOperator(SparseOperator):
         if operation not in (operation_assignment, operator.iadd):
             raise ValueError('Invalid reduction operation.')
 
-        f = 'fsr{0}_pt1_i{1}_m{2}_v{3}'.format(
+        f = 'fsr{0}_pt1_i{1}_r{2}_v{3}'.format(
             self._flib_id, self.matrix.data.index.dtype.itemsize,
             self.matrix.dtype.itemsize, out.dtype.itemsize)
         if hasattr(flib.operators, f):
@@ -902,8 +965,12 @@ class ProjectionOperator(SparseOperator):
             func(self.matrix.data.view(np.int8).ravel(), out.ravel(),
                  self.matrix.ncolmax, self.matrix.data.shape[0])
         else:
+            if self.shapeout is None:
+                shapein = (self.matrix.shape[0],)
+            else:
+                shapein = self.shapeout
             if isinstance(self.matrix, FSRMatrix):
-                x = ones(self.shapeout, self.dtype)
+                x = ones(shapein, self.dtype)
                 self.T(x, out=out, operation=operation)
             elif isinstance(self.matrix, FSRRotation2dMatrix):
                 if operation is operation_assignment:
@@ -911,11 +978,11 @@ class ProjectionOperator(SparseOperator):
                 index = self.matrix.data.index.ravel()
                 data = np.sqrt(self.matrix.data.r11**2 +
                                self.matrix.data.r21**2).ravel()
-                for i, d in izip(index, data):
+                for i, d in zip(index, data):
                     if i >= 0:
                         out[i] += d
             else:
-                x = ones(self.shapeout, self.dtype)
+                x = ones(shapein, self.dtype)
                 operation(out, self.T(x)[..., 0])
         return out
 
@@ -931,12 +998,19 @@ class ProjectionOperator(SparseOperator):
                 'This method does not make sense for FSRRotation2dMatrix spars'
                 'e storage.')
         x = np.asarray(x, order='C')
+        shapein = self.shapeout
+        if shapein is None:
+            block_size = self.matrix.block_size
+            shapein = (self.matrix.shape[0],) + \
+                      (() if block_size == 1 else (block_size,))
         shapeout = self.shapein
-        if not isinstance(self.matrix, FSRMatrix):
+        if shapeout is None:
+            shapeout = (self.matrix.shape[1],)
+        elif self.matrix.block_shape[1] > 1:
             shapeout = shapeout[:-1]
-        if x.shape != self.shapeout:
+        if x.shape != shapein:
             raise ValueError("Invalid input shape '{0}'. Expected shape is '{1"
-                             "}'.".format(x.shape, self.shapein))
+                             "}'.".format(x.shape, shapein))
         dtypeout = max(self.dtype, x.dtype)
         if out is None:
             pTx = empty(shapeout, dtypeout)
@@ -954,7 +1028,7 @@ class ProjectionOperator(SparseOperator):
         if operation not in (operation_assignment, operator.iadd):
             raise ValueError('Invalid reduction operation.')
 
-        f = 'fsr{0}_ptx_pt1_i{1}_m{2}_v{3}'.format(
+        f = 'fsr{0}_ptx_pt1_i{1}_r{2}_v{3}'.format(
             self._flib_id, self.matrix.data.index.dtype.itemsize,
             self.matrix.dtype.itemsize, dtypeout.itemsize)
         if hasattr(flib.operators, f):
@@ -973,50 +1047,6 @@ class ProjectionOperator(SparseOperator):
                 operation(pTx, self.T(x)[..., 0])
         return pTx, pT1
 
-    def restrict(self, mask):
-        """
-        Restrict the projection to a subspace defined by a mask
-        (True means that the element is kept). Indices are renumbered in-place.
-
-        """
-        mask = np.asarray(mask)
-        if mask.dtype != bool:
-            raise TypeError('The mask is not boolean.')
-        if isinstance(self.matrix, FSRMatrix):
-            expected = self.shapein
-        else:
-            expected = self.shapein[:-1]
-        if mask.shape != expected:
-            raise ValueError("Invalid shape '{}'. Expected value is '{}'.".
-                             format(mask.shape, expected))
-
-        itype = self.matrix.data.index.dtype
-        mtype = self.matrix.dtype
-        if itype in (np.int32, np.int64) and mtype in (np.float32, np.float64):
-            f = 'fsr{0}_restrict_i{1}_m{2}'.format(
-                self._flib_id, itype.itemsize, mtype.itemsize)
-            func = getattr(flib.operators, f)
-            ncol = func(self.matrix.data.view(np.int8).ravel(),
-                        mask.ravel(), self.matrix.ncolmax,
-                        self.matrix.shape[0] // self.matrix.block_size)
-        else:
-            ncol = np.sum(mask)
-            new_index = empty(mask.shape, itype)
-            new_index[...] = -1
-            new_index[mask] = np.arange(ncol, dtype=itype)
-            undef = self.matrix.data.index < 0
-            self.matrix.data.index = new_index[self.matrix.data.index]
-            self.matrix.data.index[undef] = -1
-
-        if isinstance(self.matrix, FSRMatrix):
-            shapein = ncol
-        elif isinstance(self.matrix, FSRRotation2dMatrix):
-            shapein = (ncol, 2)
-        else:
-            shapein = (ncol, 3)
-        self.matrix.shape = (self.matrix.shape[0], product(shapein))
-        self._reset(shapein=shapein)
-
 
 @orthogonal
 @inplace
@@ -1024,7 +1054,7 @@ class RollOperator(Operator):
     def __init__(self, n, axis=None, **keywords):
         Operator.__init__(self, **keywords)
         if axis is None:
-            axis = -1 if isscalarlike(n) else range(-len(n), 0)
+            axis = -1 if isscalarlike(n) else tuple(range(-len(n), 0))
         self.axis = (axis,) if isscalarlike(axis) else tuple(axis)
         self.n = (n,) * len(self.axis) if isscalarlike(n) else tuple(n)
         if len(self.axis) != len(self.n):
@@ -1033,12 +1063,12 @@ class RollOperator(Operator):
 
     def direct(self, input, output):
         output[...] = np.roll(input, self.n[0], axis=self.axis[0])
-        for n, axis in zip(self.n, self.axis)[1:]:
+        for n, axis in zip(self.n[1:], self.axis[1:]):
             output[...] = np.roll(output, n, axis=axis)
 
     def transpose(self, input, output):
         output[...] = np.roll(input, -self.n[0], axis=self.axis[0])
-        for n, axis in zip(self.n, self.axis)[1:]:
+        for n, axis in zip(self.n[1:], self.axis[1:]):
             output[...] = np.roll(output, -n, axis=axis)
 
 
