@@ -4,9 +4,7 @@ Define PackedTable classes:
 PackedTable:
   - Layout
       - LayoutGrid
-          - LayoutGridCircles
-      - LayoutVertex
-          - LayoutGridSquares
+          - LayoutGridSquare
   - Sampling
       - Pointing
           - PointingEquatorial
@@ -16,18 +14,18 @@ PackedTable:
       - SceneHealpix
 
 """
-from __future__ import division
-
-import copy
-import functools
-import inspect
-import numpy as np
-import types
+from __future__ import absolute_import, division, print_function
 from pyoperators import MPI
 from pyoperators.utils import (
     ilast_is_not, isalias, isclassattr, isscalarlike, product, split,
     tointtuple)
 from ..warnings import PySimulatorsWarning, warn
+import copy
+import functools
+import inspect
+import numpy as np
+import types
+from collections.abc import Callable
 
 __all__ = ['PackedTable']
 
@@ -102,7 +100,7 @@ class PackedTable(object):
 
     which are the 1d-collapsed indices of the following array coordinates:
 
-    >>> print [(i // 6, i % 6) for i in table.index]
+    >>> print([(i // 6, i % 6) for i in table.index])
     [(2, 0), (1, 0), (2, 1), (1, 1), (0, 1), (2, 2), (1, 2), (0, 2),
      (5, 2), (5, 1), (4, 2), (4, 1), (4, 0), (3, 2), (3, 1), (3, 0)]
 
@@ -144,7 +142,7 @@ class PackedTable(object):
             self._special_attributes += (k,)
             if v is None and isclassattr(k, type(self)):
                 continue
-            if not callable(v):
+            if not isinstance(v, Callable):
                 v = self.pack(v, copy=True)
             setattr(self, k, v)
 
@@ -245,13 +243,23 @@ class PackedTable(object):
         if key in self._reserved_attributes:
             raise AttributeError(
                 'The attribute {0!r} is not writeable.'.format(key))
-        if value is not None and not callable(value) and \
+        if value is not None and \
+           not isinstance(value, Callable) and \
            key in self._special_attributes:
             value = np.asanyarray(value)
             if value.ndim > 0 and value.shape[0] != len(self):
                 raise ValueError(
                     "The shape '{0}' is invalid. The expected first dimension "
                     "is '{1}'.".format(value.shape, len(self)))
+            elif value.ndim == 0:
+                try:
+                    old_value = object.__getattribute__(self, key)
+                    if old_value is not None and \
+                       not isinstance(old_value, Callable):
+                        old_value[...] = value
+                        return
+                except AttributeError:
+                    pass
         object.__setattr__(self, key, value)
 
     def __getattribute__(self, key):
@@ -263,7 +271,7 @@ class PackedTable(object):
             return self
         value = object.__getattribute__(self, key)
         if key not in object.__getattribute__(self, '_special_attributes') or \
-           not callable(value):
+           not isinstance(value, Callable):
             return value
         spec = inspect.getargspec(value)
         nargs = len(spec.args)
@@ -321,7 +329,7 @@ class PackedTable(object):
             for _ in self._special_attributes if _ not in ('index', 'removed')
             and not isinstance(object.__getattribute__(self, _),
                                (types.FunctionType, types.MethodType,
-                                types.NoneType)))
+                                type(None))))
         if len(attributes) > 1:
             out += '\n    '
         out += ',\n    '.join('{0}={1}'.format(_, str(getattr(self, _))[:65])
@@ -330,6 +338,15 @@ class PackedTable(object):
 
     __repr__ = __str__
 
+    def copy(self):
+        """
+        Return a deep copy of the packed array.
+
+        """
+        out = copy.deepcopy(self)
+        out.comm = self.comm.Dup()
+        return out
+
     def pack(self, x, out=None, copy=False):
         """
         Convert a multi-dimensional array into a 1-dimensional array which
@@ -435,6 +452,68 @@ class PackedTable(object):
         object.__setattr__(out, 'comm', comm)
         return out
 
+    def gather(self, *args):
+        """
+        MPI-gather the (already scattered) table or a given array.
+
+        table_global = table_local.gather()
+        array_global = table_local.gather(array_local)
+
+        Parameter
+        ---------
+        array_local : array-like, optional
+            If provided, gather the scattered input array, instead of the whole
+            table.
+
+        Returns
+        -------
+        table_global : PackedTable
+            The global packed table, whose all special attribute have been
+            MPI-gathered.
+        array_global : array
+            The MPI-gathered input array.
+
+        """
+        def func(x):
+            x = np.asarray(x)
+            out = np.empty((ntot,) + x.shape[1:], x.dtype)
+            nbytes = product(x.shape[1:]) * x.itemsize
+            self.comm.Allgatherv(
+                x.view(np.byte),
+                [out.view(np.byte),
+                 ([_ * nbytes for _ in counts],
+                  [_ * nbytes for _ in offsets])])
+            return out
+        ntot = np.array(len(self))
+        self.comm.Allreduce(MPI.IN_PLACE, ntot, op=MPI.SUM)
+        counts = []
+        offsets = [0]
+        for s in split(ntot, self.comm.size):
+            n = s.stop - s.start
+            counts.append(n)
+            offsets.append(offsets[-1] + n)
+        offsets.pop()
+
+        if len(args) == 1:
+            return func(args[0])
+        elif len(args) > 1:
+            raise TypeError(
+                'gather takes at most 1 argument ({} given)'.format(len(args)))
+
+        out = copy.copy(self)
+        out._index = self._normalize_int_selection(
+            func(self.index), product(self.shape[:self.ndim]))
+        for k in out._special_attributes:
+            if k in out._reserved_attributes:
+                continue
+            try:
+                v = self.__dict__[k]
+            except KeyError:
+                continue
+            if isinstance(v, np.ndarray) and v.ndim > 0:
+                setattr(out, k, func(v))
+        return out
+
     def split(self, n):
         """
         Split the table in partitioning groups.
@@ -669,7 +748,7 @@ class UnpackedTable(object):
         v = getattr(self._packed, key)
         if v is None:
             return v
-        if callable(v):
+        if isinstance(v, Callable):
             return lambda *args: self._packed.unpack(v(*args))
         out = self._packed.unpack(v).view()
         out.flags.writeable = False
@@ -692,7 +771,7 @@ class UnpackedTable(object):
         if key in p._reserved_attributes:
             raise AttributeError(
                 'The attribute {0!r} is not writeable.'.format(key))
-        if callable(value):
+        if isinstance(value, Callable):
             raise TypeError(
                 'A function cannot be set as an unpacked attribute.')
         object.__setattr__(p, key, p.pack(value, copy=True))
