@@ -3,26 +3,58 @@
 !
 module module_pointingmatrix
 
-    use module_math,       only : NaN, nint_down, nint_up
-    use module_projection, only : intersection_polygon_unity_square
-    use module_precision,  only : sp
-    use module_tamasis,    only : p
+    use module_math_old,  only : NaN, nint_down, nint_up
+    use module_geometry,  only : intersection_polygon_unity_square_r8
+    use module_precision, only : sp
+    use module_tamasis,   only : p
     implicit none
     private
 
     public :: PointingElement
-    public :: PointingElement_i4_m4
-    public :: PointingElement_i4_m8
-    public :: PointingElement_i8_m4
-    public :: PointingElement_i8_m8
-    public :: PointingElementRot2d_i4_m4
-    public :: PointingElementRot2d_i4_m8
-    public :: PointingElementRot2d_i8_m4
-    public :: PointingElementRot2d_i8_m8
-    public :: PointingElementRot3d_i4_m4
-    public :: PointingElementRot3d_i4_m8
-    public :: PointingElementRot3d_i8_m4
-    public :: PointingElementRot3d_i8_m8
+    public :: PointingElement_i4_r4
+    public :: PointingElement_i4_r8
+    public :: PointingElement_i8_r4
+    public :: PointingElement_i8_r8
+    public :: PointingElementBlock_1_2_i4_r4
+    public :: PointingElementBlock_1_2_i4_r8
+    public :: PointingElementBlock_1_2_i8_r4
+    public :: PointingElementBlock_1_2_i8_r8
+    public :: PointingElementBlock_1_3_i4_r4
+    public :: PointingElementBlock_1_3_i4_r8
+    public :: PointingElementBlock_1_3_i8_r4
+    public :: PointingElementBlock_1_3_i8_r8
+    public :: PointingElementBlock_2_1_i4_r4
+    public :: PointingElementBlock_2_1_i4_r8
+    public :: PointingElementBlock_2_1_i8_r4
+    public :: PointingElementBlock_2_1_i8_r8
+    public :: PointingElementBlock_2_2_i4_r4
+    public :: PointingElementBlock_2_2_i4_r8
+    public :: PointingElementBlock_2_2_i8_r4
+    public :: PointingElementBlock_2_2_i8_r8
+    public :: PointingElementBlock_2_3_i4_r4
+    public :: PointingElementBlock_2_3_i4_r8
+    public :: PointingElementBlock_2_3_i8_r4
+    public :: PointingElementBlock_2_3_i8_r8
+    public :: PointingElementBlock_3_1_i4_r4
+    public :: PointingElementBlock_3_1_i4_r8
+    public :: PointingElementBlock_3_1_i8_r4
+    public :: PointingElementBlock_3_1_i8_r8
+    public :: PointingElementBlock_3_2_i4_r4
+    public :: PointingElementBlock_3_2_i4_r8
+    public :: PointingElementBlock_3_2_i8_r4
+    public :: PointingElementBlock_3_2_i8_r8
+    public :: PointingElementBlock_3_3_i4_r4
+    public :: PointingElementBlock_3_3_i4_r8
+    public :: PointingElementBlock_3_3_i8_r4
+    public :: PointingElementBlock_3_3_i8_r8
+    public :: PointingElementRot2d_i4_r4
+    public :: PointingElementRot2d_i4_r8
+    public :: PointingElementRot2d_i8_r4
+    public :: PointingElementRot2d_i8_r8
+    public :: PointingElementRot3d_i4_r4
+    public :: PointingElementRot3d_i4_r8
+    public :: PointingElementRot3d_i8_r4
+    public :: PointingElementRot3d_i8_r8
     public :: pmatrix_direct
     public :: pmatrix_direct_one_pixel_per_sample
     public :: pmatrix_transpose
@@ -41,62 +73,222 @@ module module_pointingmatrix
         real(sp)  :: value
     end type PointingElement
 
-    type PointingElement_i4_m4
+    type PointingElement_i4_r4
         integer*4 :: index
         real*4    :: value
     end type
 
-    type PointingElement_i8_m4
+    type PointingElement_i8_r4
         integer*8 :: index
         real*4    :: value
     end type
 
-    type PointingElement_i4_m8
+    type PointingElement_i4_r8
         integer*4 :: index
         real*8    :: value
     end type
 
-    type PointingElement_i8_m8
+    type PointingElement_i8_r8
         integer*8 :: index
         real*8    :: value
     end type
 
-    type PointingElementRot2d_i4_m4
+    type PointingElementBlock_1_2_i4_r4
+        integer*4 :: index
+        real*4    :: value(2,1)
+    end type
+
+    type PointingElementBlock_1_2_i8_r4
+        integer*8 :: index
+        real*4    :: value(2,1)
+    end type
+
+    type PointingElementBlock_1_2_i4_r8
+        integer*4 :: index
+        real*8    :: value(2,1)
+    end type
+
+    type PointingElementBlock_1_2_i8_r8
+        integer*8 :: index
+        real*8    :: value(2,1)
+    end type
+
+    type PointingElementBlock_1_3_i4_r4
+        integer*4 :: index
+        real*4    :: value(3,1)
+    end type
+
+    type PointingElementBlock_1_3_i8_r4
+        integer*8 :: index
+        real*4    :: value(3,1)
+    end type
+
+    type PointingElementBlock_1_3_i4_r8
+        integer*4 :: index
+        real*8    :: value(3,1)
+    end type
+
+    type PointingElementBlock_1_3_i8_r8
+        integer*8 :: index
+        real*8    :: value(3,1)
+    end type
+
+    type PointingElementBlock_2_1_i4_r4
+        integer*4 :: index
+        real*4    :: value(1,2)
+    end type
+
+    type PointingElementBlock_2_1_i8_r4
+        integer*8 :: index
+        real*4    :: value(1,2)
+    end type
+
+    type PointingElementBlock_2_1_i4_r8
+        integer*4 :: index
+        real*8    :: value(1,2)
+    end type
+
+    type PointingElementBlock_2_1_i8_r8
+        integer*8 :: index
+        real*8    :: value(1,2)
+    end type
+
+    type PointingElementBlock_2_2_i4_r4
+        integer*4 :: index
+        real*4    :: value(2,2)
+    end type
+
+    type PointingElementBlock_2_2_i8_r4
+        integer*8 :: index
+        real*4    :: value(2,2)
+    end type
+
+    type PointingElementBlock_2_2_i4_r8
+        integer*4 :: index
+        real*8    :: value(2,2)
+    end type
+
+    type PointingElementBlock_2_2_i8_r8
+        integer*8 :: index
+        real*8    :: value(2,2)
+    end type
+
+    type PointingElementBlock_2_3_i4_r4
+        integer*4 :: index
+        real*4    :: value(3,2)
+    end type
+
+    type PointingElementBlock_2_3_i8_r4
+        integer*8 :: index
+        real*4    :: value(3,2)
+    end type
+
+    type PointingElementBlock_2_3_i4_r8
+        integer*4 :: index
+        real*8    :: value(3,2)
+    end type
+
+    type PointingElementBlock_2_3_i8_r8
+        integer*8 :: index
+        real*8    :: value(3,2)
+    end type
+
+    type PointingElementBlock_3_1_i4_r4
+        integer*4 :: index
+        real*4    :: value(1,3)
+    end type
+
+    type PointingElementBlock_3_1_i8_r4
+        integer*8 :: index
+        real*4    :: value(1,3)
+    end type
+
+    type PointingElementBlock_3_1_i4_r8
+        integer*4 :: index
+        real*8    :: value(1,3)
+    end type
+
+    type PointingElementBlock_3_1_i8_r8
+        integer*8 :: index
+        real*8    :: value(1,3)
+    end type
+
+    type PointingElementBlock_3_2_i4_r4
+        integer*4 :: index
+        real*4    :: value(2,3)
+    end type
+
+    type PointingElementBlock_3_2_i8_r4
+        integer*8 :: index
+        real*4    :: value(2,3)
+    end type
+
+    type PointingElementBlock_3_2_i4_r8
+        integer*4 :: index
+        real*8    :: value(2,3)
+    end type
+
+    type PointingElementBlock_3_2_i8_r8
+        integer*8 :: index
+        real*8    :: value(2,3)
+    end type
+
+    type PointingElementBlock_3_3_i4_r4
+        integer*4 :: index
+        real*4    :: value(3,3)
+    end type
+
+    type PointingElementBlock_3_3_i8_r4
+        integer*8 :: index
+        real*4    :: value(3,3)
+    end type
+
+    type PointingElementBlock_3_3_i4_r8
+        integer*4 :: index
+        real*8    :: value(3,3)
+    end type
+
+    type PointingElementBlock_3_3_i8_r8
+        integer*8 :: index
+        real*8    :: value(3,3)
+    end type
+
+    type PointingElementRot2d_i4_r4
         integer*4 :: index
         real*4    :: r11, r21
     end type
 
-    type PointingElementRot2d_i8_m4
+    type PointingElementRot2d_i8_r4
         integer*8 :: index
         real*4    :: r11, r21
     end type
 
-    type PointingElementRot2d_i4_m8
+    type PointingElementRot2d_i4_r8
         integer*4 :: index
         real*8    :: r11, r21
     end type
 
-    type PointingElementRot2d_i8_m8
+    type PointingElementRot2d_i8_r8
         integer*8 :: index
         real*8    :: r11, r21
     end type
 
-    type PointingElementRot3d_i4_m4
+    type PointingElementRot3d_i4_r4
         integer*4 :: index
         real*4    :: r11, r22, r32
     end type
 
-    type PointingElementRot3d_i8_m4
+    type PointingElementRot3d_i8_r4
         integer*8 :: index
         real*4    :: r11, r22, r32
     end type
 
-    type PointingElementRot3d_i4_m8
+    type PointingElementRot3d_i4_r8
         integer*4 :: index
         real*8    :: r11, r22, r32
     end type
 
-    type PointingElementRot3d_i8_m8
+    type PointingElementRot3d_i8_r8
         integer*8 :: index
         real*8    :: r11, r22, r32
     end type
@@ -488,11 +680,11 @@ contains
 
                     polygon(1,:) = coords(1,:,icoord) - (ix - 0.5_p)
                     polygon(2,:) = coords(2,:,icoord) - (iy - 0.5_p)
-                    val = abs(intersection_polygon_unity_square(polygon, nvertices))
+                    val = real(intersection_polygon_unity_square_r8(polygon, nvertices), sp)
                     if (val == 0) cycle
                     if (npixels <= npixels_per_sample) then
                         pmatrix(npixels,icoord)%index  = ix + iy * nx
-                        pmatrix(npixels,icoord)%value = val
+                        pmatrix(npixels,icoord)%value = abs(val)
                     end if
                     npixels = npixels + 1
 
